@@ -22,6 +22,6 @@
 void reset_console(void);
 int capture_start(void);
 int capture_end(void);
-int print_summary(void);
+void print_summary(void);
 
 #endif /* __CAPTURE_H__ */
