@@ -2,7 +2,7 @@
  * configuration.c: logapp configuration handling
  *
  *
- * Copyright (C) 2007-2009 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2010 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -94,6 +94,7 @@ pipe_t pstdout = {
 	.blen		= 65540,
 	.bfill		= 0,
 	.memcr		= 0,
+	.ptyremovecr	= 0,
 	.recalclen	= 0,
 	.fgcol		= -1,
 	.bgcol		= -1,
@@ -124,6 +125,7 @@ pipe_t pstderr = {
 	.blen		= 65540,
 	.bfill		= 0,
 	.memcr		= 0,
+	.ptyremovecr	= 0,
 	.recalclen	= 0,
 	.fgcol		= 1,
 	.bgcol		= -1,
@@ -548,6 +550,7 @@ static int fixup_pipe(pipe_t *pipe)
 	if (pipe->lineprefix != NULL) {
 		pipe->lineprefixlen = strlen(pipe->lineprefix);
 		if (pipe->lineprefixlen == 0) {
+			free(pipe->lineprefix);
 			pipe->lineprefix = NULL;
 		}
 	}
@@ -653,6 +656,10 @@ int fixup_config(void)
 		logfile.sizelimit = config.maxlogsize * 1024;
 	}
 
+	if (CONFIG_SUPPORT_PTY && config.usepty) {
+		app.pstdout->ptyremovecr = config.ptyremovecr;
+	}
+
 	if (fixup_pipe(app.pstdout))
 		return -1;
 
@@ -676,6 +683,14 @@ int fixup_config(void)
 	return 0;
 }
 
+void cleanup_config(void)
+{
+	if (app.argv) {
+		free(app.argv);
+		app.argv = NULL;
+	}
+}
+
 int get_display_parameters(void)
 {
 	/* As the terminal settings are not available if the stream is
@@ -836,9 +851,9 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 		return -1;
 	}
 
-	if (value == &argv[*argp][2]) {
-		/* Long parameter has no value provided - we assume 'true' for
-		 * boolean options */
+	if (argv && value == &argv[*argp][2]) {
+		/* Long parameter has no value provided - we assume 'true'
+		 * for boolean options */
 		if (arglist[argid].type == TBOOL) {
 			value = boolvalues[0].string;
 		}
@@ -846,10 +861,12 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 		error("no value expected\n");
 		return -1;
 	} else if (!value && (arglist[argid].type != TNONE))  {
-		if (*argp < argc -1) {
-			(*argp)++;
-			if (argv[*argp][0]!='-') {
-				value = argv[*argp];
+		if (argp) {
+			if (*argp < argc -1) {
+				(*argp)++;
+				if (argv && argv[*argp][0]!='-') {
+					value = argv[*argp];
+				}
 			}
 		}
 		if (!value) {
@@ -899,6 +916,10 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 				break;
 		case TSTRING:	if (arglist[argid].set > priority)
 					break;
+				if ((*((char**)arglist[argid].var) != NULL)
+				    && (arglist[argid].set > 0)) {
+					free(*((char**)arglist[argid].var));
+				}
 				*((char**)arglist[argid].var) = strdup(value);
 				if (*((char**)arglist[argid].var) == NULL) {
 					error_outofmemory();
@@ -1025,10 +1046,9 @@ int parse_args(int argc, char* argv[])
 
 #define LINEBUFSIZE	251
 
-int parse_configline(char* line)
+int parse_configline(char* line, char **appconfig_section)
 {
 	static int lineno;
-	static char* appconfig;
 	char* name;
 	char* value;
 	char* tmp;
@@ -1038,14 +1058,17 @@ int parse_configline(char* line)
 
 	/* remove comments and spaces */
 	name = line + strspn(line, " \t");
+
 	if (strlen(name) == 0)
 		return 0;
-	value = name + strcspn(name, " \t=");
-	*(value) = '\0';
-	tmp = value;
 
-	tmp = value + strcspn(value, "\t ");
-	value = value + 1 + strspn(value + 1, " \t=");
+	value = name + strcspn(name, " \t=");
+
+	*(value) = '\0';
+
+	if (strlen(value+1) > 0)
+		value = value + 1 + strspn(value + 1, " \t=");
+
 	if (*value=='\"') {
 		value++;
 		tmp = strchr(value, '\"');
@@ -1064,13 +1087,14 @@ int parse_configline(char* line)
 	*tmp = '\0';
 	
 	/* check if we change to a tagged section */
-	if (!strlen(value) && (name[0] == '[')
+	if ((strlen(value)==0) && (strlen(name)>1) && (name[0] == '[')
 		   && (name[strlen(name)-1] == ']') ) {
 		name = name + 1;
 		name[strlen(name) - 1] = '\0';
-		free(appconfig);
-		appconfig = strdup(name);
-		if (!appconfig) {
+		if (*appconfig_section != NULL)
+			free(*appconfig_section);
+		*appconfig_section = strdup(name);
+		if (!appconfig_section) {
 			error_outofmemory();
 			return -1;
 		}
@@ -1078,10 +1102,10 @@ int parse_configline(char* line)
 	}
 	
 	/* check if the section should be ignored */
-	if (appconfig != NULL) {
+	if (*appconfig_section != NULL) {
 		if (config.configsection != NULL) {
 			char* tmp;
-			if (strchr(appconfig, '/') == NULL) {
+			if (strchr(*appconfig_section, '/') == NULL) {
 				tmp = strrchr(config.configsection, '/');
 				if (tmp == NULL)
 					tmp = config.configsection;
@@ -1090,7 +1114,7 @@ int parse_configline(char* line)
 			} else {
 				tmp = config.configsection;
 			}
-			if (strcmp(appconfig, tmp)) {
+			if (strcmp(*appconfig_section, tmp)) {
 				return 0;
 			}
 		} else {
@@ -1100,12 +1124,10 @@ int parse_configline(char* line)
 
 	/* get the configuration */
 	if ((argid = get_argid(name, 0)) >= 0) {
-		int argc = 1;
-		char* argv[2];
 		if ((strlen(value) == 0) && (arglist[argid].type == TNONE)) {
 			value = NULL;
 		}
-		get_argvalue(1, argid, &argc, argc, argv, value);
+		get_argvalue(1, argid, NULL, 0, NULL, value);
 	} else {
 		warning("unknown parameter in config file (line %d)\n",
 			lineno);
@@ -1119,6 +1141,7 @@ int get_config(void)
 {
 	FILE* conffile = NULL;
 	char *filename;
+	char *appconfig_section = NULL;
 	char linebuf[LINEBUFSIZE];
 
 	/* open config file */
@@ -1157,11 +1180,16 @@ int get_config(void)
 	/* parse each line */
 	config.configfile = filename;
 	while (fgets(linebuf, LINEBUFSIZE, conffile) != NULL) {
-		linebuf[strcspn(linebuf, "#\n")]='\0';
-		if (parse_configline(linebuf))
+		linebuf[strcspn(linebuf, "#\n")] = '\0';
+		if (parse_configline(linebuf, &appconfig_section))
 			return -1;
 	}
 
+	if (appconfig_section)
+		free(appconfig_section);
+
+	fclose(conffile);
+
 	return 0;
 }
 
