@@ -2,7 +2,7 @@
  * configuration.h: logapp configuration header
  *
  *
- * Copyright (C) 2007-2009 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2010 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -85,6 +85,7 @@ typedef struct {
 	int	cfhno;	/* numerical console file handle */
 	char*   name;   /* visible pipe name */
 	pthread_t ct;   /* capture thread */
+	char*   dbuf;   /* physical data buffer pointed to by other buffers */
 	char*	buf;	/* char buffer */
 	readbuf_t rdbuf; /* used in linebased mode for faster read */
 	int	state;  /* read return state */
@@ -95,6 +96,7 @@ typedef struct {
 	int	switchtocbmode; /* escape has bin detected, now switch to 
 				   charbased mode as soon as possible */
 	int	memcr;  /* temporary stored CR */
+	int     ptyremovecr; /* remove carriage return from CR-LF */
 
 	int	blen;	/* buffer length (1 will be added for overhead) */
 	int	bfill;  /* number of characters currently buffered */
@@ -205,6 +207,7 @@ extern int parse_args(int argc, char* argv[]);
 extern void show_configuration(void);
 extern int get_config(void);
 extern int fixup_config(void);
+extern void cleanup_config(void);
 extern int get_display_parameters(void);
 extern int adjust_clipping(void);
 extern int check_for_disable_keywords(void);
