@@ -2,7 +2,7 @@
  * logapp.h: Logapp main header file
  *
  *
- * Copyright (C) 2007-2009 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2010 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,25 @@ typedef enum { MESSAGE, WARNING, ERROR } t_printtype;
 #define STRING_ERROR	" error: "
 void print(t_printtype type, const char *format,...);
 void error_outofmemory(void);
+int execcmd(char* cmd);
+
+/* These values should normally be defined by the Makefile */
+#ifndef EXECUTABLE
+# define EXECUTABLE	"logapp"
+#endif
+#ifndef VERSION
+# define VERSION	""
+#endif
+#ifndef SVN_REVISION
+# define SVN_REVISION	""
+#endif
+#ifndef CONFIG_SUPPORT_PTY
+# define CONFIG_SUPPORT_PTY 1
+#endif
+#ifndef CONFIG_USE_THREADS
+# define CONFIG_USE_THREADS 1
+#endif
+
 
 #if (CONFIG_USE_THREADS == 1)
 /* Let the system take care of making errno thread safe. */
