@@ -2,7 +2,7 @@
  * main.c: logapp main file
  *
  *
- * Copyright (C) 2007-2009 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2010 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <time.h>
 #include <stdarg.h>
 
 #include <sys/socket.h>
@@ -32,15 +31,32 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <signal.h>
+
+#if defined(__FreeBSD__)
+#include <libutil.h>
+#include <sys/ioctl.h>
+#include <termios.h>
+#elif defined(__OpenBSD__) || defined(__NetBSD__)
+#include <util.h>
+#include <termios.h>
+#elif defined(__APPLE__)
+#include <util.h>
+#else
 #include <pty.h>
+#endif
 
 #include "configuration.h"
 #include "logfile.h"
 #include "capture.h"
 
+const char string_outofmemory[] = EXECUTABLE " error: out of memory\n";
+
 void error_outofmemory(void)
 {
-	fprintf(stderr, EXECUTABLE " error: out of memory\n");
+	int ret;
+
+	ret = write(STDERR_FILENO, string_outofmemory,
+		    sizeof(string_outofmemory));
 }
 
 #define max(a, b) ((a) > (b) ? (a) : (b))
@@ -63,16 +79,17 @@ void print(t_printtype type, const char *format,...)
 	fmtstring = malloc(fmt_len);
 	if (!fmtstring) {
 		error_outofmemory();
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 	va_start(argptr, format);
 
-	if (type) {
+	if (type!=MESSAGE) {
 		memcpy(fmtstring, EXECUTABLE, exec_len);
 		fmt_len = exec_len;
 	} else {
 		fmt_len = 0;
+		*fmtstring = '\0';
 	}
 
 	switch (type) {
@@ -86,16 +103,24 @@ void print(t_printtype type, const char *format,...)
 			break;
 		default: break;
 	}
+
 	memcpy(fmtstring + fmt_len, format, format_len);
 	*(fmtstring + fmt_len + format_len) = '\0';
 
-	vfprintf(stderr, fmtstring, argptr);
+	if (vfprintf(stderr, fmtstring, argptr) < 0) {
+		exit(EXIT_FAILURE);
+	}
 
 	va_end(argptr);
 
 	free(fmtstring);
 }
 
+int execcmd(char* cmd)
+{
+	return system(cmd);
+}
+
 #define LINELENGTH 80
 #define DESCOFFSET 33
 void usage(void)
@@ -116,6 +141,9 @@ void usage(void)
 	}
 
 	for (i=0; i<arglistsize; i++) {
+		unsigned int descoffset = 0;
+		unsigned int desclen;
+
 		if ((!arglist[i].shrt) && (!arglist[i].lng)
 		    && (arglist[i].desc)) {
 			message("\n%s\n", arglist[i].desc);
@@ -141,8 +169,10 @@ void usage(void)
 
 		/* Not checking return values of snprintf above, but be sure
 		 * that offset didn't get smaller before continuing */
-		if (offset < 5)
-			exit(-1);
+		if (offset < 5) {
+			error("unable to generate usage information\n");
+			exit(EXIT_FAILURE);
+		}
 
 		for (j=offset; j<DESCOFFSET; j++) {
 			linebuf[j] = ' ';
@@ -154,8 +184,6 @@ void usage(void)
 			continue;
 		}
 
-		int descoffset = 0;
-		unsigned int desclen;
 		for (j=0; j<((desclen = strlen(arglist[i].desc)) + 2); j++) {
 			if (arglist[i].desc[j] == ' ' ||
 			    (arglist[i].desc[j] == '\n') ||
@@ -184,7 +212,8 @@ void usage(void)
 		AUTHOR);
 }
 
-void version(void) {
+void version(void)
+{
 	message("%s %s\n", EXECUTABLE, VERSION);
 	message("Build date: %s %s\n", __DATE__, __TIME__);
 	if (strlen(SVN_REVISION))
@@ -227,8 +256,14 @@ int fork_child()
 		}
 
 		if (CONFIG_SUPPORT_PTY && config.usepty) {
-			openpty(&pipe_stdout[0], &pipe_stdout[1], NULL,
-				app.ptytermios, app.ptysize);
+			if (openpty(&pipe_stdout[0], &pipe_stdout[1], NULL,
+				app.ptytermios, app.ptysize) == -1) {
+				error("opening PTY for %s failed with error "
+				      "%d\n", app.pstdout->name, errno);
+				/* no need to remove CRs if PTY cannot be
+				 * opened */
+				app.pstdout->ptyremovecr = 0;
+			}
 		} else {
 			if (pipe(pipe_stdout) < 0) {
 				error("creating stdout pipe handler\n");
@@ -256,7 +291,7 @@ int fork_child()
 		if (!config.disable) {
 			close(pipe_stdout[1]);
 			close(pipe_stderr[1]);
-			app.pstdout->fh = pipe_stdout[0]; //mpty;
+			app.pstdout->fh = pipe_stdout[0];
 			app.pstderr->fh = pipe_stderr[0];
 		}
 
@@ -399,22 +434,22 @@ int main(int argc, char *argv[])
 
 	if (signal(SIGINT, sig_handler) == SIG_ERR) {
 		error("creating signal handler\n");
-		return -1;
+		exit(EXIT_FAILURE);
 	}
 
 	if (signal(SIGQUIT, sig_handler) == SIG_ERR) {
 		error("creating signal handler\n");
-		return -1;
+		exit(EXIT_FAILURE);
 	}
 
 	if (signal(SIGTERM, sig_handler) == SIG_ERR) {
 		error("creating signal handler\n");
-		return -1;
+		exit(EXIT_FAILURE);
 	}
 
 	if (signal(SIGCHLD, sig_handler) == SIG_ERR) {
 		error("creating signal handler\n");
-		return -1;
+		exit(EXIT_FAILURE);
 	}
 
 	/* Start the application */
@@ -430,7 +465,7 @@ int main(int argc, char *argv[])
 
 	/* execute preexec command */
 	if (config.preexeccmd) {
-		int ret = system(config.preexeccmd);
+		int ret = execcmd(config.preexeccmd);
 		if (ret) {
 			print(config.exitonexecfail?ERROR:WARNING,
 			      "preexec command \"%s\" returned 0x%x\n",
@@ -450,7 +485,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* Wait for application to exit get its exit status */
-	wait(&app.exit_state);
+	waitpid(app.pid, &app.exit_state, 0);
 	
 	/* Wait for capture threads to exit */
 	if (!config.disable)
@@ -462,7 +497,7 @@ int main(int argc, char *argv[])
 
 	/* execute postexec command */
 	if (config.postexeccmd) {
-		int ret = system(config.postexeccmd);
+		int ret = execcmd(config.postexeccmd);
 		if (ret) {
 			print(config.exitonexecfail?ERROR:WARNING,
 			      "postexec command \"%s\" returned 0x%x\n",
@@ -481,6 +516,9 @@ int main(int argc, char *argv[])
 	if (config.printsummary)
 		print_summary();
 
+	/* free data allocated for configuration */
+	cleanup_config();
+
 	/* We are returning the exit status of the application if possible */
 	if (WIFEXITED(app.exit_state))
 		exit(WEXITSTATUS(app.exit_state));
